using System;
using System.Collections.Generic;
using System.Text;

namespace Google_Maps_Download
{
    class mapURL
    {
        public string MapByType(int zoom, int x, int y, string map_type)
        {
            if (map_type == "iM")
                return GoogleMap(zoom, x, y);
            if (map_type == "iT")
                return GoogleTransparent(zoom, x, y);
            if (map_type == "iS")
                return GoogleSatellite(zoom, x, y);

            if (map_type == "iMSNm")
                return MSNMap(zoom, x, y);
            if (map_type == "iMSNh")
                return MSNHybrid(zoom, x, y);
            if (map_type == "iMSNs")
                return MSNSatellite(zoom, x, y);

            if (map_type == "iASKm")
                return AskMap(zoom, x, y);
            if (map_type == "iASKt")
                return AskTransparent(zoom, x, y);
            if (map_type == "iASKs")
                return AskSatellite(zoom, x, y);

            if (map_type == "iYs")
                return YahooSatellite(zoom, x, y);

            if (map_type == "iTt")
                return TerraTopo(zoom, x, y);

            else
                return "about:null";
        }

        public string ExtByType(string map_type)
        {
            if (map_type == "iM")
                return ".png";
            if (map_type == "iT")
                return ".png";
            if (map_type == "iS")
                return ".jpg";

            if (map_type == "iMSNm")
                return ".png";
            if (map_type == "iMSNh")
                return ".jpg";
            if (map_type == "iMSNs")
                return ".jpg";

            if (map_type == "iASKm")
                return ".png";
            if (map_type == "iASKt")
                return ".png";
            if (map_type == "iASKs")
                return ".jpg";

            if (map_type == "iYs")
                return ".jpg";

            if (map_type == "iTt")
                return ".jpg";

            else
                return ".err";
        }

        private Random random = new Random();

        private string RandomGoogle()
        {
            int ran = random.Next(-1, 3);
            return ran.ToString().Replace("-1", "");
        }

        private string RandomMSN()
        {
            int ran = random.Next(0, 3);
            return ran.ToString();
        }

        private string RandomAsk()
        {
            int ran = random.Next(1, 4);
            return ran.ToString();
        }

        public string GoogleMap(int zoom, int x, int y)
        {
            return "http://mt" + RandomGoogle() + ".google.com/mt?n=404&v=w2.21&x=" + x.ToString() + "&y=" + y.ToString() + "&zoom=" + zoom.ToString();
        }

        public string GoogleTransparent(int zoom, int x, int y)
        {
            return "http://mt" + RandomGoogle() + ".google.com/mt?n=404&v=w2t.22&x=" + x.ToString() + "&y=" + y.ToString() + "&zoom=" + zoom.ToString();
        }

        public string GoogleSatellite(int zoom, int x, int y)
        {
            return "http://kh" + RandomGoogle() + ".google.com/kh?n=404&v=9&t=" + CoordsToKeyhole(x, y, zoom);
        }

        
        public string MSNMap(int zoom, int x, int y)
        {
            string msn_loc = CoordsToMSN(x, y, zoom);
            return "http://r" + RandomMSN() + ".ortho.tiles.virtualearth.net/tiles/r" + msn_loc + ".png?g=25";
        }

        public string MSNHybrid(int zoom, int x, int y)
        {
            string msn_loc = CoordsToMSN(x, y, zoom);
            return "http://h" + RandomMSN() + ".ortho.tiles.virtualearth.net/tiles/h" + msn_loc + ".jpeg?g=25";
        }

        public string MSNSatellite(int zoom, int x, int y)
        {
            string msn_loc = CoordsToMSN(x, y, zoom);
            return "http://a" + RandomMSN() + ".ortho.tiles.virtualearth.net/tiles/a" + msn_loc + ".jpeg?g=25";
        }


        private string CoordsToMSN(int x, int y, int zoom)
        {
            string to_return = CoordsToKeyhole(x, y, zoom);

            to_return = to_return.Substring(1, to_return.Length - 1);

            to_return = to_return.Replace("q", "0").Replace("r", "1").Replace("t", "2").Replace("s", "3");

            return to_return;
        }

        #region borrowed code
        private string CoordsToKeyhole(int x, int y, int zoom)
        {
            String s = "";
            double myX = x;
            double myY = y;

            for (int i = 17; i > zoom; i--)
            {
                double rx = (fmod(myX, 2));
                myX = Math.Floor(myX / 2);
                double ry = (fmod(myY, 2));
                myY = Math.Floor(myY / 2);
                s = getKeyholeDirection(rx, ry) + s;
            }
            return 't' + s;
        }

        private double fmod(double myX, int i)
        {
            return myX % i;
        }

        public String getKeyholeDirection(double x, double y)
        {
            if (x == 1)
            {
                if (y == 1)
                {
                    return "s";
                }
                else if (y == 0)
                {
                    return "r";
                }
            }
            else if (x == 0)
            {
                if (y == 1)
                {
                    return "t";
                }
                else if (y == 0)
                {
                    return "q";
                }
            }

            return "";
        }
        #endregion

        public string AskMap(int zoom, int x, int y)
        {
            int mt = System.Convert.ToInt32(Math.Pow(2, 17 - zoom)); // (mt is the max number of tiles at this level - 16 for google's zoom level 13, 8 for 14, and so on) 
            int Az = zoom + 2;
            int Ax = x - mt / 2;
            int Ay = y - mt / 2;
            return "http://mapstatic" + RandomAsk() + ".ask.com/map/" + Az.ToString() + "/" + Ax.ToString() + "/" + Ay.ToString();
        }

        public string AskTransparent(int zoom, int x, int y)
        {
            int mt = System.Convert.ToInt32(Math.Pow(2, 17 - zoom)); // (mt is the max number of tiles at this level - 16 for google's zoom level 13, 8 for 14, and so on) 
            int Az = zoom + 2;
            int Ax = x - mt / 2;
            int Ay = y - mt / 2;
            return "http://mapstatic" + RandomAsk() + ".ask.com/mapt/" + Az.ToString() + "/" + Ax.ToString() + "/" + Ay.ToString();
        }

        public string AskSatellite(int zoom, int x, int y)
        {
            int mt = System.Convert.ToInt32(Math.Pow(2, 17 - zoom)); // (mt is the max number of tiles at this level - 16 for google's zoom level 13, 8 for 14, and so on) 
            int Az = zoom + 2;
            int Ax = x - mt / 2;
            int Ay = y - mt / 2;
            return "http://mapstatic" + RandomAsk() + ".ask.com/sat/" + Az.ToString() + "/" + Ax.ToString() + "/" + Ay.ToString();
        }

        public string YahooSatellite(int zoom, int x, int y)
        {
            // http://us.maps3.yimg.com/aerial.maps.yimg.com/img?md=200605101500&x=18&y=7&z=12&v=1.5&t=a // the date is not required it seems, and there are no other servers
            int Yz = zoom + 1;
            int Yx = x;
            int Yy = System.Convert.ToInt32(Math.Pow(2, 16 - zoom)) - y - 1;
            return "http://us.maps3.yimg.com/aerial.maps.yimg.com/img?x=" + Yx.ToString() + "&y=" + Yy.ToString() + "&z=" + Yz.ToString() + "&v=1.5&t=a";
        }

        public string TerraTopo(int zoom, int x, int y)
        {
            // these approximate LA
            int Tl = 9 - zoom;
            int Tx = System.Convert.ToInt32(Math.Pow(2, Tl)) * 33 + x;
            int Ty = System.Convert.ToInt32(Math.Pow(2, Tl)) * 256 - y;
            return "http://nww.terraserver-usa.com/nwwtile.ashx?T=102&L=" + Tl.ToString() + "&X=" + Tx.ToString() + "&Y=" + Ty.ToString();

        }

        // NASA
        /* 
         * Blue Marble Next Generation imagery
         * 
         * // START SAME south pole as 0 from dayline (diff 0,0) - 4 vert spans world at zoom 0
         * Blue Marble 200401 to 200412
         * http://worldwind25.arc.nasa.gov/tile/tile.aspx?T=bmng.topo.200409&L=4&X=44&Y=51
         * 
         * Blue Marble 200401 to 200412 (Bathymetry)
         * http://worldwind25.arc.nasa.gov/tile/tile.aspx?T=bmng.topo.bathy.200412&L=4&X=41&Y=52
         * // END SAME
         * 
         * // START SAME 'eq' as 0 from 'dayline' (diff 0,0) - 0 is quite zoomed in
         * Landsat 7 imagery
         * 
         * NLT Landsat (Visible & Pseudo Color)
         * 
         * NLT LandSat7 (Visible Color)
         * http://worldwind25.arc.nasa.gov/tile/tile.aspx?T=105&L=3&X=297&Y=442
         * 
         * NLT LandSat7 (Pseudo Color)
         * http://worldwind28.arc.nasa.gov/TestWebApp/WebForm1.aspx?T=nlt_pseudo&L=4&X=589&Y=883
         * 
         * Geocover 1990 & 2000 (pseudo)
         * 
         * NLT LandSat7 (Pseudo Color) GeoCover 2000
         * http://worldwind25.arc.nasa.gov/tile/tile.aspx?T=geocover2000&L=1&X=73&Y=110
         * 
         * NLT LandSat7 (Pseudo Color) GeoCover 1990
         * http://worldwind28.arc.nasa.gov/TestWebApp/WebForm1.aspx?T=es_1990jpg&L=2&X=147&Y=220
         * // END SAME
         * 
         * Landsat 7 imagery
         * OnEarth (visible & pseudo) -- where?
         * 
         * http://worldwind25.arc.nasa.gov/tile/tile.aspx?T=bmng.topo.200409&L=4&X=27&Y=54
         * MATCHES!!! (diff L)
         * http://worldwind25.arc.nasa.gov/tile/tile.aspx?T=105&L=0&X=27&Y=54
         * // 
         * http://worldwind25.arc.nasa.gov/tile/tile.aspx?T=105&L=2&X=109&Y=219
         * COMPLETE MISMATCH
         * http://nww.terraserver-usa.com/nwwtile.ashx?T=102&L=1&X=153&Y=308
         * 
         * 
         * http://worldwind25.arc.nasa.gov/tile/tile.aspx?T=105&L=2&X=109&Y=219
         * 'MATCH'? - no
         * http://mt1.google.com/mt?n=404&v=w2.21&x=87&y=205&zoom=8
         * or
         * http://mt1.google.com/mt?n=404&v=w2.21&x=175&y=411&zoom=7
         * 'MATCH'? - not really either
         * http://nww.terraserver-usa.com/nwwtile.ashx?T=102&L=2&X=308&Y=616
         * 
         * 
         * 
         * USGS imagery
         * 
         * Digital Ortho (monochrome declassified NRO KH-11 spy satellite photographs)
         * 
         * // START SAME 'eq' as 0 from 'dayline' (diff 0,0) - quite zoomed in
         * USGS Digital Ortho 1m
         * http://nww.terraserver-usa.com/nwwtile.ashx?T=101&L=0&X=100&Y=150
         * http://nww.terraserver-usa.com/nwwtile.ashx?T=101&L=1&X=209&Y=311
         * http://nww.terraserver-usa.com/nwwtile.ashx?T=101&L=2&X=417&Y=621
         * 
         * Topographic maps
         * 
         * USGS Topo Maps
         * http://nww.terraserver-usa.com/nwwtile.ashx?T=102&L=0&X=100&Y=150
         * http://nww.terraserver-usa.com/nwwtile.ashx?T=102&L=1&X=209&Y=311
         * http://nww.terraserver-usa.com/nwwtile.ashx?T=102&L=2&X=417&Y=621
         * // END SAME
         * 
         * Urban Area Ortho (montaged color aerial photography of many major US metropolitan areas)
         * 
         * // SAME BUT DOWN?
         * USGS Urban Area Ortho
         * http://nww.terraserver-usa.com/nwwtile.ashx?T=104&L=2&X=416&Y=621
         * 
         * 
         * Zoomit! imagery (community produced layer)
         * 
         * ZoomIt! Data
         * LINZ (montaged color aerial photography of New Zealand)
         * http://madmappers.tileservice.worldwindcentral.org/getTile?T=nz_linz2001_2.5m&version=1&interface=worldwind&firstLevel=11&L=3&X=15968&Y=2030
         * 
         * GSWA (Topographic and geological maps of Western Australia)
         * FIND!!!
         * 
         * US imagery (montaged color aerial photography of many major US metropolitan areas)
         * FIND!!!
         * 
         * South Africa Robben Island 0.5 m
         * FIND!!!
         * 
         */

    }
}
